unit example02main;

// Code Example 02 of using XDOM 2.2.1
// Delphi 3 Implementation
//
// You need XDOM 2.2.9 or above to use this source code.
// The latest version of XDOM can be found at "http://www.philo.de/xml/".
//
// Copyright (c) 2000 by Dieter Khler
// ("http://www.philo.de/homepage.htm")
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

interface

uses
  XDOM,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Tabs, FileCtrl, ComCtrls, ExtCtrls, Grids;

type
  TMainpage = class(TForm)
    Label3: TLabel;
    Label4: TLabel;
    OpenDialog1: TOpenDialog;
    SpeedButton1: TSpeedButton;
    XmlToDomParser1: TXmlToDomParser;
    DomImplementation1: TDomImplementation;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label1: TLabel;
    SpeedButton2: TSpeedButton;
    TabControl1: TTabControl;
    Memo1: TMemo;
    TabSet1: TTabSet;
    StringGrid1: TStringGrid;
    StringGrid2: TStringGrid;
    Memo2: TMemo;
    procedure OpenFile(Sender: TObject);
    procedure XmlToDomParser1ExternalSubset(Sender: TObject;
      const PublicId, SystemId: WideString; var extSubset: WideString);
    procedure TabSet1Click(Sender: TObject);
    procedure CloseFile(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Mainpage: TMainpage;

implementation

{$R *.DFM}

procedure TMainpage.OpenFile(Sender: TObject);
var
  UpTime: integer;
  Doc: TdomDocument;
  index,i: integer;
begin
  OpenDialog1.InitialDir:= ExtractFileDir(Label1.caption);
  if OpenDialog1.Execute then begin

    Label8.caption:= '';
    Label10.caption:= '';

    if not FileExists (OpenDialog1.FileName) then begin
      Label3.caption:= '';
      memo2.text:= 'File not found!';
      exit;
    end;

    Memo1.Clear;
    Memo1.Update;

    with XmlToDomParser1 do begin
      try
        UpTime:= GetTickCount;
        Doc:= FileToDom(OpenDialog1.FileName);
        Label3.caption:= IntToStr(GetTickCount-UpTime);
        index:= TabSet1.Tabs.Add(ExtractFileName(Doc.Filename));
        TabSet1.TabIndex:= index;
      except
        Label3.caption:= '';
      end; {try ... except ...}
    end; {with ...}

    Memo1.Update;
    TabSet1.Update;
    with memo2 do begin
      clear;
      with XmlToDomParser1.ErrorHandler.errorList do begin
        for i:= 0 to count-1 do
          memo2.Lines.Add(TXmlParserError(Items[i]).GetErrorStr(en));
      end;
      if text = '' then text:= 'Document successfully parsed.';
      update;
    end;
  end; {if OpenDialog1.Execute ...}
end;

procedure TMainpage.XmlToDomParser1ExternalSubset(Sender: TObject;
  const PublicId, SystemId: WideString; var extSubset: WideString);
var
  path: TFilename;
  fileAsStringlist: TStringlist;
begin
  // Quick and dirty: This works only with DTD's in the
  // same directory as the document file.
  extSubset:= '';
  Label8.caption:= PublicId;
  Label10.caption:= SystemId;
  if systemId <> '' then begin
    fileAsStringlist:= TStringlist.create;
    try
      path := concat(ExtractFileDir(OpenDialog1.FileName),'\',SystemID);
      fileAsStringlist.LoadFromFile(path);
      extSubset:= fileAsStringlist.Text;
    finally
      fileAsStringlist.free;
    end;
  end;
end;

procedure TMainpage.TabSet1Click(Sender: TObject);
var
  doc: TdomDocument;
  i: integer;
begin
  with Memo1 do begin
    clear;
    Update;
  end;
  with StringGrid1 do begin
    RowCount:= 0;
    Cells[0,0]:= '';
    Cells[1,0]:= '';
    Update;
  end;
  with StringGrid2 do begin
    RowCount:= 0;
    Cells[0,0]:= '';
    Cells[1,0]:= '';
    Update;
  end;
  if TabSet1.TabIndex > -1 then begin
    SpeedButton2.Enabled:= true;
    doc:= (XmlToDomParser1.DOMImpl.documents.Item(TabSet1.TabIndex) as TdomDocument);
    Label1.caption:= doc.Filename;
    with Memo1 do begin
      Text:= Doc.CodeAsString;  // := FDocumentSCList[TabSet1.TabIndex];
      Update;
    end;
    if assigned(doc.Doctype) then begin
      with StringGrid1 do begin
        RowCount:= doc.Doctype.Entities.Length;
        for i:= 0 to doc.Doctype.Entities.Length-1 do begin
          Cells[0,i]:= doc.Doctype.Entities.item(i).nodename;
          Cells[1,i]:= TdomEntity(doc.Doctype.Entities.item(i)).value;
        end;
        Update;
      end;
      with StringGrid2 do begin
        RowCount:= doc.Doctype.ParameterEntities.Length;
        for i:= 0 to doc.Doctype.ParameterEntities.Length-1 do begin
          Cells[0,i]:= doc.Doctype.ParameterEntities.item(i).nodename;
          Cells[1,i]:= TdomParameterEntity(doc.Doctype.ParameterEntities.item(i)).nodevalue;
        end;
        Update;
      end;
    end;
  end else Label1.caption:= '';
end;

procedure TMainpage.CloseFile(Sender: TObject);
var
  doc: TdomDocument;
begin
  if TabSet1.TabIndex > -1 then begin
    with TabSet1 do begin
      doc:= (XmlToDomParser1.DOMImpl.documents.Item(TabIndex) as TdomDocument);
      XmlToDomParser1.DOMImpl.FreeDocument(doc);
      Tabs.Delete(TabIndex);
      if Tabs.Count = 0 then SpeedButton2.Enabled:= false;
    end;
  end;
end;

procedure TMainpage.TabControl1Change(Sender: TObject);
begin
  case TabControl1.TabIndex of
    0: begin Memo1.show; StringGrid1.hide; StringGrid2.hide; end;
    1: begin Memo1.hide; StringGrid1.show; StringGrid2.hide; end;
    2: begin Memo1.hide; StringGrid1.hide; StringGrid2.show; end;
  end;
end;

end.
